@@ -7,7 +7,7 @@ if (!defined('GLPI_ROOT')) {
 /**
  * ---------------------------------------------------------------------
  * GLPI - Gestionnaire Libre de Parc Informatique
- * Copyright (C) 2015-2018 Teclib' and contributors.
+ * Copyright (C) 2015-2022 Teclib' and contributors.
  *
  * http://glpi-project.org
  *
@@ -55,7 +55,7 @@ class Item_OperatingSystem extends CommonDBRelation {
             if ($_SESSION['glpishow_count_on_tabs']) {
                $nb = self::countForItem($item);
             }
-            return self::createTabEntry(_n('Operating system', 'Operating systems', Session::getPluralNumber()), $nb);
+            return self::createTabEntry(OperatingSystem::getTypeName(Session::getPluralNumber()), $nb);
       }
       return '';
    }
@@ -66,42 +66,22 @@ class Item_OperatingSystem extends CommonDBRelation {
    }
 
    /**
-    * Print the item's operating system form
+    * Get operating systems related to a given item
     *
-    * @param CommonDBTM $item Item instance
+    * @param CommonDBTM $item  Item instance
+    * @param string     $sort  Field to sort on
+    * @param string     $order Sort order
     *
-    * @since 9.2
-    *
-    * @return Nothing (call to classes members)
-   **/
-   static function showForItem(CommonDBTM $item, $withtemplate = 0) {
-      global $DB, $CFG_GLPI;
+    * @return DBmysqlIterator
+    */
+   public static function getFromItem(CommonDBTM $item, $sort = null, $order = null): DBmysqlIterator {
+      global $DB;
 
-      //default options
-      $params = ['rand' => mt_rand()];
-
-      $columns = [
-         __('Name'),
-         __('Version'),
-         __('Architecture'),
-         __('Service pack')
-      ];
-
-      if (isset($_GET["order"]) && ($_GET["order"] == "ASC")) {
-         $order = "ASC";
-      } else {
-         $order = "DESC";
+      if ($sort === null) {
+         $sort = "glpi_items_operatingsystems.id";
       }
-
-      if ((isset($_GET["sort"]) && !empty($_GET["sort"]))
-         && isset($columns[$_GET["sort"]])) {
-         $sort = "`".$_GET["sort"]."`";
-      } else {
-         $sort = "`glpi_items_operatingsystems`.`id`";
-      }
-
-      if (empty($withtemplate)) {
-         $withtemplate = 0;
+      if ($order === null) {
+         $order = 'ASC';
       }
 
       $iterator = $DB->request([
@@ -145,12 +125,53 @@ class Item_OperatingSystem extends CommonDBRelation {
          ],
          'ORDERBY'   => "$sort $order"
       ]);
+      return $iterator;
+   }
 
+   /**
+    * Print the item's operating system form
+    *
+    * @param CommonDBTM $item Item instance
+    *
+    * @since 9.2
+    *
+    * @return void
+   **/
+   static function showForItem(CommonDBTM $item, $withtemplate = 0) {
+      global $DB;
+
+      //default options
+      $params = ['rand' => mt_rand()];
+
+      $columns = [
+         __('Name'),
+         _n('Version', 'Versions', 1),
+         _n('Architecture', 'Architectures', 1),
+         OperatingSystemServicePack::getTypeName(1)
+      ];
+
+      if (isset($_GET["order"]) && ($_GET["order"] == "ASC")) {
+         $order = "ASC";
+      } else {
+         $order = "DESC";
+      }
+
+      if ((isset($_GET["sort"]) && !empty($_GET["sort"]))
+         && isset($columns[$_GET["sort"]])) {
+         $sort = $_GET["sort"];
+      } else {
+         $sort = "glpi_items_operatingsystems.id";
+      }
+
+      if (empty($withtemplate)) {
+         $withtemplate = 0;
+      }
+
+      $iterator = self::getFromItem($item, $sort, $order);
       $number = count($iterator);
       $i      = 0;
 
       $os = [];
-      $used = [];
       while ($data = $iterator->next()) {
          $os[$data['assocID']] = $data;
       }
@@ -208,7 +229,7 @@ class Item_OperatingSystem extends CommonDBRelation {
       }
 
       foreach ($columns as $key => $val) {
-         $header_end .= "<th".($sort == "`$key`" ? " class='order_$order'" : '').">".
+         $header_end .= "<th".($sort == $key ? " class='order_$order'" : '').">".
                         "<a href='javascript:reloadTab(\"sort=$key&amp;order=".
                           (($order == "ASC") ?"DESC":"ASC")."&amp;start=0\");'>$val</a></th>";
       }
@@ -273,8 +294,6 @@ class Item_OperatingSystem extends CommonDBRelation {
 
 
    function showForm($ID, $options = []) {
-      global $DB;
-
       $colspan = 4;
 
       echo "<div class='center'>";
@@ -285,7 +304,7 @@ class Item_OperatingSystem extends CommonDBRelation {
       $rand = mt_rand();
 
       echo "<tr class='headerRow'><th colspan='".$colspan."'>";
-      echo __('Operating system');
+      echo OperatingSystem::getTypeName(1);
       echo Html::hidden('itemtype', ['value' => $this->fields['itemtype']]);
       echo Html::hidden('items_id', ['value' => $this->fields['items_id']]);
       echo "</th></tr>";
@@ -295,26 +314,26 @@ class Item_OperatingSystem extends CommonDBRelation {
       echo "<td>";
       OperatingSystem::dropdown(['value' => $this->fields["operatingsystems_id"], 'rand' => $rand]);
       echo "</td>";
-      echo "<td><label for='dropdown_operatingsystemversions_id$rand'>".__('Version')."</label></td>";
+      echo "<td><label for='dropdown_operatingsystemversions_id$rand'>"._n('Version', 'Versions', 1)."</label></td>";
       echo "<td >";
       OperatingSystemVersion::dropdown(['value' => $this->fields["operatingsystemversions_id"], 'rand' => $rand]);
       echo "</td>";
       echo "</tr>";
 
       echo "<tr class='tab_bg_1'>";
-      echo "<td><label for='dropdown_operatingsystemarchitectures_id$rand'>".__('Architecture')."</label></td>";
+      echo "<td><label for='dropdown_operatingsystemarchitectures_id$rand'>"._n('Architecture', 'Architectures', 1)."</label></td>";
       echo "<td >";
       OperatingSystemArchitecture::dropdown(['value'
                                                  => $this->fields["operatingsystemarchitectures_id"], 'rand' => $rand]);
       echo "</td>";
-      echo "<td><label for='dropdown_operatingsystemservicepacks_id$rand'>".__('Service pack')."</label></td>";
+      echo "<td><label for='dropdown_operatingsystemservicepacks_id$rand'>".OperatingSystemServicePack::getTypeName(1)."</label></td>";
       echo "<td >";
       OperatingSystemServicePack::dropdown(['value'
                                                  => $this->fields["operatingsystemservicepacks_id"], 'rand' => $rand]);
       echo "</td></tr>";
 
       echo "<tr class='tab_bg_1'>";
-      echo "<td><label for='dropdown_operatingsystemkernelversions_id$rand'>"._n('Kernel', 'kernel', 1)."</label></td>";
+      echo "<td><label for='dropdown_operatingsystemkernelversions_id$rand'>"._n('Kernel', 'Kernels', 1)."</label></td>";
       echo "<td >";
       OperatingSystemKernelVersion::dropdown([
          'value'  => $this->fields['operatingsystemkernelversions_id'],
@@ -323,7 +342,7 @@ class Item_OperatingSystem extends CommonDBRelation {
       ]);
       echo "</td>";
 
-      echo "<td><label for='dropdown_operatingsystemeditions_id$rand'>" . __('Edition') . "</label></td>";
+      echo "<td><label for='dropdown_operatingsystemeditions_id$rand'>" . _n('Edition', 'Editions', 1) . "</label></td>";
       echo "<td >";
       OperatingSystemEdition::dropdown([
          'value'  => $this->fields['operatingsystemeditions_id'],
@@ -340,12 +359,12 @@ class Item_OperatingSystem extends CommonDBRelation {
       echo "<td><label for='textfield_license_number$rand'>".__('Serial number')."</label></td>";
       echo "<td >";
       Html::autocompletionTextField($this, 'license_number', ['rand' => $rand]);
-      echo "</td><td colspan='2'></td></tr>";
+      echo "</td></tr>";
       $options['formfooter'] = false;
       $this->showFormButtons($options);
    }
 
-   function getRawName() {
+   protected function computeFriendlyName() {
       $item = getItemForItemtype($this->fields['itemtype']);
       $item->getFromDB($this->fields['items_id']);
       $name = $item->getTypeName(1) . ' ' . $item->getName();
@@ -357,6 +376,8 @@ class Item_OperatingSystem extends CommonDBRelation {
    /**
     * Duplicate operating system from an item template to its clone
     *
+    * @deprecated 9.5
+    *
     * @param string  $itemtype    itemtype of the item
     * @param integer $oldid       ID of the item to clone
     * @param integer $newid       ID of the item cloned
@@ -367,6 +388,7 @@ class Item_OperatingSystem extends CommonDBRelation {
    static function cloneItem($itemtype, $oldid, $newid, $newitemtype = '') {
       global $DB;
 
+      Toolbox::deprecated('Use clone');
       $iterator = $DB->request([
          'FROM'   => self::getTable(),
          'WHERE'  => [
@@ -398,16 +420,6 @@ class Item_OperatingSystem extends CommonDBRelation {
          'name'               => __('Characteristics')
       ];
 
-      $tab[] = [
-         'id'                 => '1',
-         'table'              => $this->getTable(),
-         'field'              => 'name',
-         'name'               => __('Name'),
-         'datatype'           => 'itemlink',
-         'massiveaction'      => false,
-         'autocomplete'       => true,
-      ];
-
       $tab[] = [
          'id'                 => '2',
          'table'              => $this->getTable(),
@@ -460,7 +472,7 @@ class Item_OperatingSystem extends CommonDBRelation {
          'id'                 => '46',
          'table'              => 'glpi_operatingsystemversions',
          'field'              => 'name',
-         'name'               => __('Version'),
+         'name'               => _n('Version', 'Versions', 1),
          'datatype'           => 'dropdown',
          'massiveaction'      => false,
          'joinparams'         => [
@@ -478,7 +490,7 @@ class Item_OperatingSystem extends CommonDBRelation {
          'id'                 => '41',
          'table'              => 'glpi_operatingsystemservicepacks',
          'field'              => 'name',
-         'name'               => __('Service pack'),
+         'name'               => OperatingSystemServicePack::getTypeName(1),
          'datatype'           => 'dropdown',
          'massiveaction'      => false,
          'joinparams'         => [
@@ -522,7 +534,7 @@ class Item_OperatingSystem extends CommonDBRelation {
          'id'                 => '61',
          'table'              => 'glpi_operatingsystemarchitectures',
          'field'              => 'name',
-         'name'               => __('Architecture'),
+         'name'               => _n('Architecture', 'Architectures', 1),
          'datatype'           => 'dropdown',
          'massiveaction'      => false,
          'joinparams'         => [
@@ -581,7 +593,7 @@ class Item_OperatingSystem extends CommonDBRelation {
          'id'                 => '63',
          'table'              => 'glpi_operatingsystemeditions',
          'field'              => 'name',
-         'name'               => __('Edition'),
+         'name'               => _n('Edition', 'Editions', 1),
          'datatype'           => 'dropdown',
          'massiveaction'      => false,
          'joinparams'         => [
@@ -604,14 +616,7 @@ class Item_OperatingSystem extends CommonDBRelation {
 
       $specificities              = parent::getRelationMassiveActionsSpecificities();
 
-      $specificities['itemtypes'] = [
-         'Computer',
-         'Monitor',
-         'NetworkEquipment',
-         'Peripheral',
-         'Phone',
-         'Printer'
-      ];
+      $specificities['itemtypes'] = $CFG_GLPI['operatingsystem_types'];
       return $specificities;
    }
    static function showMassiveActionsSubForm(MassiveAction $ma) {
@@ -632,11 +637,11 @@ class Item_OperatingSystem extends CommonDBRelation {
       Dropdown::showFromArray(
          'os_field', [
             'OperatingSystem'             => __('Name'),
-            'OperatingSystemVersion'      => __('Version'),
-            'OperatingSystemArchitecture' => __('Architecture'),
-            'OperatingSystemKernel'       => __('Kernel'),
-            'OperatingSystemKernelVersion'=> __('Kernel version'),
-            'OperatingSystemEdition'      => __('Edition')
+            'OperatingSystemVersion'      => _n('Version', 'Versions', 1),
+            'OperatingSystemArchitecture' => _n('Architecture', 'Architectures', 1),
+            'OperatingSystemKernel'       => OperatingSystemKernel::getTypeName(1),
+            'OperatingSystemKernelVersion'=> OperatingSystemKernelVersion::getTypeName(1),
+            'OperatingSystemEdition'      => _n('Edition', 'Editions', 1)
          ], [
             'display_emptychoice'   => true,
             'rand'                  => $rand
